#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['beersmith_direct']

package_data = \
{'': ['*']}

install_requires = \
['aracnid-config',
 'aracnid-logger',
 'aracnid-utils',
 'beautifulsoup4',
 'hjson',
 'i-mongodb',
 'xmltodict']

setup(name='beersmith_direct',
      version='1.0.1',
      description='A set of functions to retrieve and save Beersmith data into MongoDB.',
      author='Jason Romano',
      author_email='aracnid@gmail.com',
      url='https://github.com/lakeannebrewhouse/beersmith-direct',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='>=3.9',
     )
